package pp.s1184725.boppi.test;

import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.*;

import java.io.IOException;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.List;
import java.util.logging.*;

import org.apache.commons.lang3.tuple.Pair;
import org.junit.Test;

import pp.s1184725.boppi.*;

public class CheckerTest {
	static final Path directory = Paths.get("src/pp/s1184725/boppi/test/parsing/");
	private List<LogRecord> log;

	static private List<LogRecord> checkAndGetLog(String code) {
		Logger logger = Logger.getAnonymousLogger();
		Pair<BasicParser, List<LogRecord>> pair = BasicParserHelper.getParserWithLog(code, logger);
		new BoppiBasicChecker(logger, new Annotations()).visit(pair.getLeft().program());
		return pair.getRight();
	}

	static private List<LogRecord> checkAndGetLog(Path code) throws IOException {
		Logger logger = Logger.getAnonymousLogger();
		Pair<BasicParser, List<LogRecord>> pair = BasicParserHelper.getParserWithLog(code, logger);
		new BoppiBasicChecker(logger, new Annotations()).visit(pair.getLeft().program());
		return pair.getRight();
	}




	@Test
	public void correctExpressionTest() throws Exception {
		log = checkAndGetLog(directory.resolve("simpleExpression.boppi"));
		assertThat(log, empty());
	}

	@Test
	public void wrongExpressionTest() {
		log = checkAndGetLog("+true");
		assertThat(log, hasSize(1));
		assertThat(log.get(0).getLevel(), is(Level.SEVERE));

		log = checkAndGetLog("5 || true");
		assertThat(log, hasSize(1));
		assertThat(log.get(0).getLevel(), is(Level.SEVERE));

		log = checkAndGetLog("6 + 'c'");
		assertThat(log, hasSize(1));
		assertThat(log.get(0).getLevel(), is(Level.SEVERE));

		log = checkAndGetLog("4 + print(5, 6)");
		assertThat(log, hasSize(1));
		assertThat(log.get(0).getLevel(), is(Level.SEVERE));

		log = checkAndGetLog("print(print(3, 5))");
		assertThat(log, hasSize(1));
		assertThat(log.get(0).getLevel(), is(Level.SEVERE));
	}

	@Test
	public void correctVariableTest() throws Exception {
		log = checkAndGetLog(directory.resolve("simpleVariable.boppi"));
		assertThat(log, empty());
	}

	@Test
	public void wrongVariableTest() {
		log = checkAndGetLog("var bool name; name := 5");
		assertThat(log, hasSize(1));
		assertThat(log.get(0).getLevel(), is(Level.SEVERE));

		log = checkAndGetLog("undefinedName");
		assertThat(log, hasSize(1));
		assertThat(log.get(0).getLevel(), is(Level.SEVERE));

		log = checkAndGetLog("var undefinedType name; 1");
		assertThat(log, hasSize(1));
		assertThat(log.get(0).getLevel(), is(Level.SEVERE));

		log = checkAndGetLog("var bool endsWithDeclaration;");
		assertThat(log, hasSize(1));
		assertThat(log.get(0).getLevel(), is(Level.SEVERE));

		log = checkAndGetLog("var bool var1; var var1 var2; var2 := 'c' ");
		assertThat(log, hasSize(1));
		assertThat(log.get(0).getLevel(), is(Level.SEVERE));
	}

	@Test
	public void correctScopeTest() throws Exception {
		log = checkAndGetLog(directory.resolve("simpleScope.boppi"));
		assertThat(log, empty());
	}

	@Test
	public void wrongScopeTest() {
		log = checkAndGetLog("var bool var1; var bool var1; 1");
		assertThat(log, hasSize(1));
		assertThat(log.get(0).getLevel(), is(Level.SEVERE));

		log = checkAndGetLog("var bool var1; var char var1; 1");
		assertThat(log, hasSize(1));
		assertThat(log.get(0).getLevel(), is(Level.SEVERE));

		log = checkAndGetLog("{ var int var1; var1 := 4}; var int var2; var1");
		assertThat(log, hasSize(1));
		assertThat(log.get(0).getLevel(), is(Level.SEVERE));
	}

	@Test
	public void correctConditionalTest() throws Exception {
		log = checkAndGetLog(directory.resolve("if.boppi"));
		assertThat(log, empty());

		log = checkAndGetLog(directory.resolve("while.boppi"));
		assertThat(log, empty());
	}
}
