package pp.s1184725.boppi.test;

import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.containsString;
import static org.hamcrest.Matchers.empty;
import static org.hamcrest.Matchers.hasSize;
import static org.hamcrest.Matchers.is;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.nio.charset.StandardCharsets;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.List;
import java.util.logging.LogRecord;
import java.util.logging.Logger;

import org.antlr.v4.runtime.ParserRuleContext;
import org.apache.commons.lang3.tuple.Pair;
import org.junit.Test;

import pp.iloc.Simulator;
import pp.s1184725.boppi.Annotations;
import pp.s1184725.boppi.BasicParser;
import pp.s1184725.boppi.BasicParserHelper;
import pp.s1184725.boppi.BoppiBasicChecker;
import pp.s1184725.boppi.BoppiBasicGenerator;

public class GeneratorTest {
	static final Path directory = Paths.get("src/pp/s1184725/boppi/test/parsing/");
	private Pair<String, List<LogRecord>> result;

	static private Pair<String, List<LogRecord>> generateAndGetLog(String code, String input) {
		Logger logger = Logger.getAnonymousLogger();
		Pair<BasicParser, List<LogRecord>> pair = BasicParserHelper.getParserWithLog(code, logger);
		return derp(logger, pair, input);
	}
	
	static private Pair<String, List<LogRecord>> generateAndGetLog(Path code, String input) throws IOException {
		Logger logger = Logger.getAnonymousLogger();
		Pair<BasicParser, List<LogRecord>> pair = BasicParserHelper.getParserWithLog(code, logger);
		return derp(logger, pair, input);
	}
	
	static private Pair<String, List<LogRecord>> derp(Logger logger, Pair<BasicParser, List<LogRecord>> pair, String input) {
		Annotations annotater = new Annotations();
		ParserRuleContext tree = pair.getLeft().program();
		new BoppiBasicChecker(logger, annotater).visit(tree);
		BoppiBasicGenerator generator = new BoppiBasicGenerator(annotater);
		generator.visit(tree);
		Simulator s = new Simulator(generator.prog);
		InputStream in = new ByteArrayInputStream(input.getBytes(StandardCharsets.UTF_8));
		ByteArrayOutputStream out = new ByteArrayOutputStream();
		s.setIn(in);
		s.setOut(out);
		try {
			s.run();
		}
		catch (Exception e) {
			logger.severe(e.getMessage());
		}
		return Pair.of(out.toString().replaceAll("\r\n", "\n"), pair.getRight());
	}
	
	
	
	
	@Test
	public void correctExpressionTest() throws IOException {
		result = generateAndGetLog(directory.resolve("simpleExpression.boppi"), "");
		assertThat(result.getRight(), empty());
	}
	
	@Test
	public void wrongExpressionTest() {
		result = generateAndGetLog("1/0", "");
		assertThat(result.getRight(), hasSize(1));
		assertThat(result.getRight().get(0).getMessage(), containsString("zero"));
	}

	@Test
	public void correctVariableTest() throws IOException {
		result = generateAndGetLog(directory.resolve("simpleVariable.boppi"), "");
		assertThat(result.getRight(), empty());
	}

	@Test
	public void basicPrograms() throws IOException {
		result = generateAndGetLog("print(5*3)", "");
		assertThat(result.getRight(), empty());
		assertThat(result.getLeft(), is("15\n"));
		
		result = generateAndGetLog("print('T', 'e', 's', 't', '!')", "");
		assertThat(result.getRight(), empty());
		assertThat(result.getLeft(), is("T\ne\ns\nt\n!\n"));
		
		result = generateAndGetLog("var int x; var int y; x := 3*(y := 4); print(x,y)", "");
		assertThat(result.getRight(), empty());
		assertThat(result.getLeft(), is("12\n4\n"));
		
		result = generateAndGetLog(directory.resolve("basicProgram1.boppi"), "1\nT\n");
		assertThat(result.getRight(), empty());
		assertThat(result.getLeft(), is("T\nT\n"));
		
		result = generateAndGetLog(directory.resolve("fibonacciIterative.boppi"), "6\n");
		assertThat(result.getRight(), empty());
		assertThat(result.getLeft(), is("13\n"));
	}
}
